import os
import json
from pathlib import Path

# Get the folder where this script is located.
folder = Path(__file__).parent

# This list will hold custom component registration lines to be written to item_components.js.
custom_component_lines = []

# Helper function to extract the simple item id (without namespace)
def get_simple_id(item_id):
    return item_id.split(":")[-1]

# Iterate over all JSON files in the folder.
for file in folder.glob("*.json"):
    with open(file, encoding="utf-8") as f:
        try:
            data = json.load(f)
        except json.JSONDecodeError as e:
            print(f"Skipping {file.name}: error parsing JSON ({e})")
            continue

    modified = False

    # 1. Update format version to "1.21.60"
    if data.get("format_version") != "1.21.60":
        data["format_version"] = "1.21.60"
        modified = True

    # Process only if file contains an item definition.
    if "minecraft:item" in data:
        item = data["minecraft:item"]
        description = item.get("description", {})

        # Save the full item ID.
        full_item_id = description.get("identifier", "default")
        simple_id = get_simple_id(full_item_id)

        # 2. Remove old 'category' and add new 'menu_category'
        if "category" in description:
            del description["category"]
            modified = True
        description["menu_category"] = {
            "category": "items",
            "group": "minecraft:itemGroup.name.items"
        }
        item["description"] = description
        modified = True

        # 3. Process components.
        components = item.get("components", {})

        # Remove creative category if present.
        if "minecraft:creative_category" in components:
            components.pop("minecraft:creative_category")
            modified = True

        # Update icon.
        icon = components.get("minecraft:icon")
        if isinstance(icon, dict):
            # If it's an object with a "texture" key, use that; otherwise, fall back to simple_id.
            components["minecraft:icon"] = icon.get("texture", simple_id)
            modified = True
        elif not icon:
            # If no icon exists, set it to the simple id.
            components["minecraft:icon"] = simple_id
            modified = True

        # Replace foil with glint.
        if "minecraft:foil" in components:
            components["minecraft:glint"] = components.pop("minecraft:foil")
            modified = True

        # Remove knockback resistance, if present.
        if "minecraft:knockback_resistance" in components:
            components.pop("minecraft:knockback_resistance")
            modified = True

        # Remove use duration.
        if "minecraft:use_duration" in components:
            components.pop("minecraft:use_duration")
            modified = True

        # Process food component (if present).
        food = components.get("minecraft:food")
        if food:
            # Save the cooldown_time if specified.
            cooldown = food.pop("cooldown_time", None)
            # Keep only the "can_always_eat" flag.
            components["minecraft:food"] = {"can_always_eat": food.get("can_always_eat", True)}
            # Add use_modifiers
            components["minecraft:use_modifiers"] = {
                "movement_modifier": 1.0,
                "use_duration": 99999
            }
            modified = True

            # Add cooldown component if a cooldown_time was provided.
            if cooldown:
                components["minecraft:cooldown"] = {
                    "category": simple_id,
                    "duration": cooldown // 20
                }
                modified = True

        # 4. Process events for custom components and extract add_effect info.
        events = item.pop("events", None)
        if events:
            effects_to_add = []
            # Support two cases:
            # Case 1: The event name is "add_effect" and has a "sequence"
            # Case 2: An event has an "add_effect" inside its structure.
            for event_name, event_data in events.items():
                if event_name == "add_effect" and "sequence" in event_data:
                    sequence = event_data["sequence"]
                elif isinstance(event_data, dict) and "add_effect" in event_data:
                    sequence = event_data["add_effect"].get("sequence", [])
                else:
                    continue
                for effect_entry in sequence:
                    if "add_mob_effect" in effect_entry:
                        mob_effect = effect_entry["add_mob_effect"]
                        if "effect" in mob_effect:
                            effects_to_add.append({
                                "effect": mob_effect["effect"],
                                "duration": mob_effect.get("duration", 1),
                                "amplifier": mob_effect.get("amplifier", 0)
                            })

            if effects_to_add:
                # Add custom component entry to the components.
                if "minecraft:custom_components" not in components:
                    components["minecraft:custom_components"] = [full_item_id]
                # Build the registration lines for the JavaScript file.
                effect_lines = []
                for effect in effects_to_add:
                    eff_name = effect["effect"]
                    eff_duration = effect["duration"] * 20  # Convert duration to ticks.
                    eff_amp = effect["amplifier"]
                    effect_lines.append(f"            source.addEffect('{eff_name}', {eff_duration}, {{ showParticles: true, amplifier: {eff_amp} }});")
                registration_line = (
                    f"    eventData.itemComponentRegistry.registerCustomComponent('{full_item_id}', {{\n"
                    "        onConsume(e) {\n"
                    "            const { source, itemStack } = e;\n" +
                    "\n".join(effect_lines) + "\n"
                    "        }\n"
                    "    });"
                )
                custom_component_lines.append(registration_line)
                modified = True

            # Always remove the events (even if no valid effects were found).
            modified = True

        # If any components were updated, write them back.
        if modified:
            item["components"] = components
            with open(file, "w", encoding="utf-8") as f:
                json.dump(data, f, indent=2)
            print(f"Updated: {file.name}")

# 5. Write the JavaScript file for custom components if any were generated.
if custom_component_lines:
    js_file_path = folder / "item_components.js"
    with open(js_file_path, "w", encoding="utf-8") as js_file:
        js_file.write('import { world, system } from "@minecraft/server"\n\n')
        js_file.write("world.beforeEvents.worldInitialize.subscribe(eventData => {\n")
        for line in custom_component_lines:
            js_file.write(line + "\n")
        js_file.write("});\n")
    print("Generated/Updated item_components.js")
else:
    print("No custom components generated. Check if any items have 'add_effect' events with valid effects.")
